#####
###  N(ew)MIMETools - a replacement for the MIMETools
##   Copyright (C) 2002	Maik Jablonski (maik.jablonski@uni-bielefeld.de)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#####

__doc__=''' NMIMETools Product Initialization '''

from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String
from cStringIO import StringIO
import string, mimetools

MIMEVersion = 'Mime-Version: 1.0'
MIMEError = 'MIME Tag Error'

# This is used for nested nmimes-tags

boundaryStack=[]
boundaryStack.append(mimetools.choose_boundary())

def mimeCheckArguments(args):
    ''' This function checks the arguments given to nmime- or
        nboundary-tags.
	Code is taken unmodified from the original MIMETools. '''
	
    has_key=args.has_key

    if has_key('type_expr'):
        if has_key('type'):
            raise ParseError, _tm('type and type_expr given', 'mime')
        args['type_expr']=Eval(args['type_expr'])
    elif not has_key('type'):
        args['type']='application/octet-stream'

    if has_key('disposition_expr'):
        if has_key('disposition'):
            raise ParseError, _tm('disposition and disposition_expr given', 'mime')
        args['disposition_expr']=Eval(args['disposition_expr'])
    elif not has_key('disposition'):
        args['disposition']=''

    if has_key('encode_expr'):
        if has_key('encode'):
           raise ParseError, _tm('encode and encode_expr given', 'mime')
        args['encode_expr']=Eval(args['encode_expr'])
    elif not has_key('encode'):
        args['encode']='base64'

    if has_key('name_expr'):
        if has_key('name'):
            raise ParseError, _tm('name and name_expr given', 'mime')
        args['name_expr']=Eval(args['name_expr'])
    elif not has_key('name'):
        args['name']=''

    if has_key('filename_expr'):
        if has_key('filename'):
            raise ParseError, _tm('filename and filename_expr given', 'mime')
        args['filename_expr']=Eval(args['filename_expr'])
    elif not has_key('filename'):
        args['filename']=''

    if has_key('skip_expr'):
        args['skip_expr']=Eval(args['skip_expr'])
		 
    if args['encode'] not in \
      ('base64', 'quoted-printable', 'uuencode', 'x-uuencode',
      'uue', 'x-uue', '7bit'):
        raise MIMEError, ('An unsupported encoding was specified in tag')

    return(args)


def mimeRender(self,md):
    ''' This function renders a MIME- or Boundary-Part.
        It's partly based on original MIMETools-code for evaluating the 
	arguments. '''
    
    mimeResult=''
    MIMEBoundary=boundaryStack[-1]
    
    for x in self.sections:
        a, b = x
        has_key=a.has_key

        if has_key('skip_expr') and a['skip_expr'].eval(md):
            continue

        mimeResult+='\n--%s\n' % (MIMEBoundary)

        if has_key('type_expr'): t=a['type_expr'].eval(md)
        else: t=a['type']

        if has_key('disposition_expr'): d=a['disposition_expr'].eval(md)
        else: d=a['disposition']

        if has_key('encode_expr'): e=a['encode_expr'].eval(md)
        else: e=a['encode']

        if has_key('name_expr'): n=a['name_expr'].eval(md)
        else: n=a['name']

        if has_key('filename_expr'): f=a['filename_expr'].eval(md)
        else: f=a['filename']

        if d:
            if f: mimeResult+='Content-Disposition: %s;\n filename="%s"\n' % (d, f)
            else: mimeResult+='Content-Disposition: %s\n' % (d)

        mimeResult+='Content-Transfer-Encoding: %s\n' % (e)
        
        if n: mimeResult+='Content-type: %s;\n name="%s"\n' % (t,n)
	else: mimeResult+='Content-type: %s\n' % (t)
	    
	renderedBlocks = render_blocks(b, md)
	
	if renderedBlocks[:len(MIMEVersion)]<>MIMEVersion:
		mimeResult+='\n'
		
        if e == '7bit':
            mimeResult+=renderedBlocks
        else:
            output = StringIO()
            mimetools.encode(StringIO(renderedBlocks), output, e)
            output.seek(0)
	    mimeResult+=output.read()

    return mimeResult


class NewMIMETag:
    ''' NewMIMETag - <dtml-nmime>...</dtml-nmime>.
        Same arguments as the MIMETOOLS-mime-tag. '''

    name='nmime'
    blockContinuations=()

    def __init__(self, blocks):
        self.sections = []

        for tname, args, section in blocks:
            args = parse_params( args
                                   , type=None, type_expr=None
                                   , disposition=None, disposition_expr=None
                                   , encode=None, encode_expr=None
                                   , name=None, name_expr=None
                                   , filename=None, filename_expr=None
                                   , skip_expr=None
                                   , multipart=None
                                   )
            self.multipart = args.get('multipart', 'mixed')
            self.sections.append((mimeCheckArguments(args), section.blocks))

    def render(self, md):	
	boundaryStack.append(mimetools.choose_boundary())
	MIMEBoundary=boundaryStack[-1]
	mimeResult='%s\n' % (MIMEVersion)
	mimeResult+='Content-type: multipart/%s;\n boundary="%s"\n' % (self.multipart,MIMEBoundary)
	mimeResult+=mimeRender(self,md)
	mimeResult+='\n--%s--\n' % (MIMEBoundary)
      	boundaryStack.pop()
        return mimeResult

    __call__=render

class NewBoundaryTag:
    ''' NewBoundaryTag - <dtml-nboundary>...</dtml-nboundary>.
        Same arguments as the MIMETOOLS-boundary-tag. '''

    name='nboundary'
    blockContinuations=()

    def __init__(self, blocks):
        self.sections = []

        for tname, args, section in blocks:
            args = parse_params( args
                                   , type=None, type_expr=None
                                   , disposition=None, disposition_expr=None
                                   , encode=None, encode_expr=None
                                   , name=None, name_expr=None
                                   , filename=None, filename_expr=None
                                   , skip_expr=None
                                   )
            self.sections.append((mimeCheckArguments(args), section.blocks))

    def render(self, md):
        return mimeRender(self,md)

    __call__=render

String.commands['nmime'] = NewMIMETag
String.commands['nboundary'] = NewBoundaryTag
